#include "WiimExpert.h"
#include "MotorConstants.h"
#include <msg.h>

#include <iostream>
#include <math.h>
#include <string>
#include <stdlib.h>

#define MAXSTR	200

#define M	0.00075
#define Q	-0.38437
#define D	20.5
#define PI 3.14159265358979323846

using namespace std;

// la parola extern non dichiara un nuovo identificatore, 
// ma dice "La variabile intera a e` dichiarata da qualche altra parte, 
// lascia solo lo spazio per risolvere il riferimento"

extern int MessageLineNum;
extern struct wiimote_t* wm0;

WiimExpert::WiimExpert(ModuleDispatch* kernel,int period, Spykee* s, SonarExpert* se) : 
  StringModuleMember(kernel,period,"WiimExpert")
{
  spykee = s;
  sonar = se;
  printf("\nWiimExpert built");
  fflush(stdout);
};

WiimExpert::~WiimExpert()
{
  fclose(fp);
  fclose(f);
  printf("\nWiimExpert destroyed");
  fflush(stdout);
};

void WiimExpert::RunInit()
{
	int i=0;
	
	charged=0;
	Hit=0;
	gHome=0;
	restartflag=0;
	rotflag=0;
	fps=0;
	random_sign=2;
	angoloOdo = STARTANGLE;
	
	settimer(0);
	settimer(1);
	settimer(3);
	settimer(10);
	
	center.x=0;
	center.y=0;
	home.x=STARTX;
	home.y=STARTY;
	home.a=STARTANGLE;
	goal.x=GOALX;
	goal.y=GOALY;
	rob.x=home.x;
	rob.y=home.y;
	rob.a=home.a;
	
	robotpoint=0;
	humanpoint=0;
	pointrobotflag=0;
	
	maxx=-2000;
	maxy=-2000;
	minx=2000;
	miny=2000;
	
	for(i=0;i<IRCENTERFILTER;i++){
		ir_centerarray_x[i].v=0;
		ir_centerarray_x[i].w=0;
		ir_centerarray_y[i].v=0;
		ir_centerarray_y[i].w=0;
	}
	
	//AddMessageRequest( MSG_POSITION,LOCAL );
	//AddMessageRequest( MSG_FROM_MOTION,LOCAL );
	
	wiim = new Wiim(); //inizializza (connessione al wiimote)
	
	struct timeval now;
	gettimeofday(&now,NULL);
    	base_time=now.tv_sec;

	playing = false;
	clock = 0;
	settimer(6);
  	 
  	printf("\nWiimExpert initialized");
  	fflush(stdout);
};

/* ***********************************
   
   DoYourDuty()

   *********************************** */
   

void WiimExpert::RunDuty()
{
	int i=0, x1, x2, y1, y2, x, y, a, an;
	float dir;
	float acc;
	static int positioninvalid=0;
	
	ppoint ir_center;
	float ir_centerx,ir_centery;
	int ir_centervalid;
	weightfloat newdata;
	weightfloat ir_newcenterx,ir_newcentery;
	float ir_center_distance;
	
	string str;
	unsigned char* buffer;
	
	if (elapsed(1) > 1000000)
	{
		if (clock == 0)
		{
			clock = 1;
		}
		else
		{
			clock = 0;
		}
		settimer(1);
	}
	
	if (humanpoint == 3)
	{
		printf("\n--- *** PLAYER WON *** ---");
		return;
	}
	if (robotpoint == 3)
	{
		printf("\n--- *** ROBOT WON *** ---");
/*		NewMessage(MSG_TO_BRIAN,WIIMOTE_DATA);
		AddBrianData("Hit", Hit, 0.9);
		AddBrianData("Clock", clock, 0.9);
		CloseMessage();
		SendAllStringMessages();*/
		return;
	}

	if ((elapsed(6) > 5000000) && (playing == false))
	{
		printf("\n--- *** START PLAY *** ---");
		wiim->WeakRumble(1);
		playing = true;
		Hit = 0;
		charged=0;
		settimer(5); // Timer per il tempo di durata del gioco

		NewMessage( MSG_TO_BRIAN, VISION_DATA );
		mCurrentMessage << "<D>Status " << SEARCH << " " << 1 << "</D>\n";
		CloseCurrentMessage();
		SendAllStringMessages();
	}
	
	// --- blocco d'emergenza ---
	if (wiim->getEmergencyStop()){
		NewMessage(MSG_TO_MOTION,WIIMOTE_DATA);
		mCurrentMessage<<"<C>TanSpeed 0.0</C>\n"<<"<C>RotSpeed 0.0</C>\n" << "</MESSAGE>\n" << '\0';
		SendAllStringMessages();
		return;
	}

	
		// --- messaggi da inviare direttamente ai motori --- 
	if(!(wiim->IsAuto())){
		str="";
		str=wiim->getToMotor();
		if (str!=""){
			NewMessage(MSG_TO_MOTION,WIIMOTE_DATA);
			//*wiim* da modificare, questo comunque è un messaggio valido
			// si potrebbero usare le funzioni in StringMessageManager 
			mCurrentMessage<< "<C>"<<str<<"</C>\n";
			CloseMessage();
			SendAllStringMessages();
		}else{
			NewMessage(MSG_TO_MOTION,WIIMOTE_DATA);
			mCurrentMessage<<"<C>TanSpeed 0.0</C>\n"<<"<C>RotSpeed 0.0</C>\n" << "</MESSAGE>\n" << '\0';
			SendAllStringMessages();
		}
		return;
	}
	
	if (playing == false)
	{
		NewMessage(MSG_TO_BRIAN,WIIMOTE_DATA);
		AddBrianData("Hit", Hit, 0.9);
		AddBrianData("Clock", clock, 0.9);
		CloseMessage();
		SendAllStringMessages();
		return;
	}
	
	int events = wiim->run();

	if((wm0==NULL) || (!events))
	{
		return;
	}
	
	struct timeval now;
	float t;
  	gettimeofday(&now,NULL);
  	//printf("%f       %f\n",base_time,rob.x);
  	t=now.tv_sec-base_time;		
	
	ir_center=wiim->getcenter();
	if (ir_center.valid==0){
		newdata.w=0;
		ir_newcenterx=getFilteredData(newdata,ir_centerarray_x,IRCENTERFILTER,IRCENTERATT,NULL);
		ir_newcentery=getFilteredData(newdata,ir_centerarray_y,IRCENTERFILTER,IRCENTERATT,NULL);
		if(ir_newcenterx.w==0 || ir_newcentery.w==0){
			ir_centervalid=0;
		}
	}else{
		newdata.w=1;
		newdata.v=ir_center.x;
		ir_newcenterx=getFilteredData(newdata,ir_centerarray_x,IRCENTERFILTER,IRCENTERATT,NULL);
		newdata.v=ir_center.y;
		ir_newcentery=getFilteredData(newdata,ir_centerarray_y,IRCENTERFILTER,IRCENTERATT,NULL);
		ir_centervalid=1;
	}
	ir_centerx=ir_newcenterx.v;
	ir_centery=ir_newcentery.v;	
	if(ir_centervalid==1){
		ir_center_distance=sqrt(pow((ir_centerx-1024/2),2)+pow((ir_centery-768/2),2));
	}
	
	struct timeval now2;
	float t2;
  	gettimeofday(&now2,NULL);	
	
	//printf("-- %d --- %d --- %d\n",now.tv_sec,now.tv_usec,elapsed(0));
	if(elapsed(0)>1000000){
		settimer(0);
		printf("\nWiimExpert fps:%d     ",fps);
		printstatus();
		fps=0;
	}else{
		fps++;
	}
	
	// --- messaggi al log ---
	NewMessage(MSG_TO_LOG,WIIMOTE_DATA);
	if(ir_centervalid){
		mCurrentMessage << "valid 1 xcenter "<<ir_centerx<<" ycenter "<<ir_centery;
	}else{
		mCurrentMessage << "valid 0 xcenter 0 ycenter 0";
	}
	//printf("%d %f %f \n",ir_centervalid,ir_centerx,ir_centery);
	CloseMessage();
	SendAllStringMessages();
	
	NewMessage(MSG_TO_BRIAN,WIIMOTE_DATA);
	//meccanica gioco
	
	/*if (wiim->getHButtonA()){
		wiim->showPoint(humanpoint-robotpoint);
	}*/
	
	//carica
	wiim->SetCharge(charged);
	if (ir_centervalid){
		//printf("--- %f ---\n",d);
		if(ir_center_distance<IRCENTERLIMIT){
			if(charged<5){
				charged += 0.01;
			}
		}else{
			if(charged >=0){
				charged -= 0.01;
			}
		}
	}else{
		if(charged >=0){
			charged -= 0.01;
		}
	}
	//printf("%f\n",charged);
	if(charged>4){
		if(wiim->getButtonB())// && ir_center_distance<IRCENTERHIT)
		{
			if(Hit==0){ //solo se sono visibili 2 luci?
				/*(MSG_TO_MOTION,WIIMOTE_DATA);
				mCurrentMessage<<"<C>TanSpeed 0.0</C>\n"<<"<C>RotSpeed 0.0</C>\n" << "</MESSAGE>\n" << '\0';
				SendAllStringMessages();*/
				Hit=1;
				charged=0;
				printf("\n--- *** COLPITO! *** ---");
				humanpoint++;
				playing = false;
				//spykee->soundAllarm();
				wiim->SetpointDiff(humanpoint-robotpoint);
				wiim->BlinkRight(1);
				wiim->StrongRumble(1);
				wiim->SetpointDiff(humanpoint-robotpoint);
				settimer(6);
			}
		}
	}
	if(wiim->getButtonB()){
		if(charged>2){
			wiim->WeakRumble(1);
		}
		charged=0;
	}

	//messaggio di colpito (per brian)
	/*if(!playing){
		AddBrianData("Hit",1,0.9);
	}
	else
	{
		AddBrianData("Hit",0,0.9);
	}*/
	
	//punto robot
	if ((elapsed(5) > 60000000) && (Hit == 0))
	{
		robotpoint++;
		printf("\n--- *** punto per il robot *** ---");
		playing = false;
		//spykee->soundBomb();
		Hit = -1;
		wiim->SetpointDiff(humanpoint-robotpoint);
		wiim->BlinkLeft(1);
		wiim->StrongRumble(1);
		settimer(6);
	}	
	
	//messaggi accellerazione
	acc = wiim->GetAcceleration();
	acc -=1;
	acc= fabs(acc);
	
	if (elapsed(3)>1200000){
		settimer(3);
		random_sign = rand() % 4;
		printf("\nRandom: %d", random_sign);
	}
	
	if (acc > 0.15){
		//printf("moving %f ...\n",acc);
		AddBrianData("GAcc",acc*10,1);
	}else{
		AddBrianData("GAcc",0,1);
	}
	
	AddBrianData("Random",random_sign,1);
	//AddBrianData("WiiMotePitch",wiim->getPitch(),0.9);
	AddBrianData("Hit", Hit, 1);
	AddBrianData("Clock", clock, 1);

	AddBrianData("SonarLeft", sonar->getDistance(6), 1);
	AddBrianData("SonarRight", sonar->getDistance(4), 1);
	AddBrianData("SonarBack", sonar->getDistance(0), 1);
	AddBrianData("SonarFront", sonar->getDistance(2), 1 );
//	AddBrianData("SonarNorthWest", sonar->getDistance(
	
	if ( elapsed(10) > 500000 ){
		AddBrianData( "SonarLeftOld", m_oldLeft, 1 );
		AddBrianData( "SonarRightOld", m_oldRight, 1 );
		m_oldRight = sonar->getDistance(6);
		m_oldLeft = sonar->getDistance(4);
		settimer(10);
	}

	//messaggio distanza e direzione dal centro IR (per brian)
	
	if(ir_centervalid){
		if(ir_centerx<1024/2){
			AddBrianData("IRCenter",-1*ir_center_distance,1);
		}else{
			AddBrianData("IRCenter",+1*ir_center_distance,1);
		}
	}
	
	//chiudo e invio
	CloseMessage();
	SendAllStringMessages();
}


void WiimExpert::RunClose()
{
  printf("\nWiimExpert closed");
  fflush(stdout);
};

void WiimExpert::AddBrianData(char* name,float value,float rel)
{
	float v;
	float r;
	
	v=value;
	r=rel;
	if(fabs(v)<0.001){ //evito che vengano mandati valori in notazione scientifica
		v=0;
	}
	if(fabs(r)<0.001){
		v=0;
	}
	
	mCurrentMessage<<"<D>"<<name<<" "<<v<<" "<<r<<" </D>\n";
	return;
	
}

long WiimExpert::elapsed(int i)
{ 
	struct timeval now;
	long elapsed_usec = 0;

	gettimeofday(&now, NULL);

	if (now.tv_usec > 1000000) {
		now.tv_usec -= 1000000;
		++now.tv_sec;
	}

	if (now.tv_sec > last_render[i].tv_sec)
		elapsed_usec = ((now.tv_sec - last_render[i].tv_sec) * 1000000);

	if (now.tv_usec > last_render[i].tv_usec)
		elapsed_usec += now.tv_usec - last_render[i].tv_usec;
	else
		elapsed_usec -= last_render[i].tv_usec - now.tv_usec;


	return elapsed_usec ;

}

void WiimExpert::settimer(int i)
{
	gettimeofday(&(last_render[i]),NULL);
	return;
}

void::WiimExpert::printstatus()
{
	printf("carica: %f\t",charged); 
	if(Hit){
		printf("Colpito!\t");
	}
	if(gHome){
		printf("goto Home\t");
	}
	if(rotflag){
		printf("rotflag\t");
	}
	printf(" hum:%d rob:%d\t",humanpoint,robotpoint);
	printf("\n");
}

void::WiimExpert::setRotDirection(int dir)
{
	//rotDirection = dir;
}







		

